local ADDON_NAME="AVR"
local L = LibStub("AceLocale-3.0"):GetLocale(ADDON_NAME, true)

local _G = _G
local pairs = pairs
local ipairs = ipairs
local remove = table.remove
local insert = table.insert
local max = math.max
local min = math.min
local abs = math.abs
local atan2 = math.atan2
local sin = math.sin
local cos = math.cos
local type = type
local select = select
local next = next
local GetTime = GetTime
local deg2rad = math.pi/180.0
local pi = math.pi
local format = string.format

local Core=AVR

AVRScene = {Embed = Core.Embed}
local T=AVRScene

function T:New(threed)
	if self ~= T then return end
	local s={}
	
	T:Embed(s)

	s.threed=threed
	s.meshes={}
	s.selection=0
	
	s.visible=true
	s.name=L["Scene"]
	s.from=nil
	s.id=0
	s.zone=nil
	s.shareChannel="Raid"
	s.shareTarget=""
	
	s.sceneManager=nil
	
	s.optionsTable=nil

	return s
end

function T:Pack()
	local s={
		name=self.name,
		visible=self.visible,
		from=self.from,
		id=self.id,
		selection=self.selection,
		zone=self.zone,
		shareChannel=self.shareChannel,
		shareTarget=self.shareTarget,
		meshes={}
	}
	for i=1,#self.meshes do
		insert(s.meshes,self.meshes[i]:Pack())
	end
	return s
end

function T:Unpack(s)
	self.name=s.name or L["Scene"]
	self.visible=s.visible
	if self.visible==nil then self.visible=true end
	self.from=s.from
	self.id=s.id or 0
	self.selection=s.selection or 0
	self.zone=s.zone
	self.shareChannel=s.shareChannel or "RAID"
	self.shareTarget=s.shareTarget or ""
	self.meshes={}
	for i=1,#s.meshes do
		local class=s.meshes[i].c
		local m
		if AVRMeshClasses[class] then
			m=AVRMeshClasses[class]:New()
			m:Unpack(s.meshes[i])
			insert(self.meshes,m)
			m:SetScene(self)
		else
			Core:Print(format(L["Trying to unpack scene, unknown class %s"],class))
		end
	end
	
	self.optionsTable=nil
	Core:OptionsChanged()
end

function T:SetSceneManager(m)
	self.sceneManager=m
end

function T:SelectScene()
	if not self.sceneManager then return end
	self.sceneManager:SetSelectedScene(self)
end

function T:DrawScene()
	for i=1,#self.meshes do
		self.meshes[i]:OnUpdate(self.threed)
		self.threed:DrawMesh(self.meshes[i])
	end
end

function T:AddMesh(mesh,select,selectScene)
	mesh:SetScene(self)
	insert(self.meshes,mesh)
	if select==nil or select then
		self.selection=#self.meshes
	end
	if selectScene==nil or selectScene then
		self:SelectScene()
	end
	
	self.optionsTable=nil
	Core:OptionsChanged()
	return mesh
end

function T:RemoveMesh(mesh)
	local sel=self:GetSelectedMesh()
	for i=1,#self.meshes do
		if self.meshes[i]==mesh then
			table.remove(self.meshes,i)
			i=#self.meshes
		end
	end
	if sel then self:SetSelectedMesh(sel) end
	
	self.optionsTable=nil
	Core:OptionsChanged()
end

function T:GetLastMesh()
	if #self.meshes==0 then return nil end
	return self.meshes[#self.meshes]
end

function T:SetSelectedMeshNum(num)
	if num==nil or num<0 or num>#self.meshes then num=0 end
	self.selection=num
end

function T:SetSelectedMesh(mesh)
	for i=1,#self.meshes do
		if self.meshes[i]==mesh then
			self.selection=i
			return
		end
	end
	self.selection=0
end

function T:GetSelectedMesh()
	if self.selection<=0 then return nil
	else return self.meshes[self.selection] end
end

function T:MakeOwnCopy()
	local p=self:Pack()
	local scene=AVRScene:New(self.threed)
	scene:Unpack(p)
	self.sceneManager:AddScene(scene)
end

function T:ClearScene()
	self.selection=0
	for i=1,#self.meshes do
		self.meshes[i].removed=true
	end
	self.meshes={}
	
	self.optionsTable=nil
	Core:OptionsChanged()
end

function T:Remove()
	self.sceneManager:RemoveScene(self)
end
